<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
if ( ! class_exists( 'UBP_Woo_Box_Product_Controller' ) ) {
	class UBP_Woo_Box_Product_Controller {

		public function __construct() {
			 add_action( 'wp_ajax_ubp_load_box_product_items', array( $this, 'ubp_load_products' ) );
			add_action( 'wp_ajax_nopriv_ubp_load_box_product_items', array( $this, 'ubp_load_products' ) );

			/**
			 * Categories filter scripts start
			 */
			add_action( 'wp_ajax_nopriv_ubp_caf_category_changed', array( $this, 'ubp_caf_category_changed' ) );
			add_action( 'wp_ajax_ubp_caf_category_changed', array( $this, 'ubp_caf_category_changed' ) );

			add_action( 'wp_ajax_nopriv_ubp_load_box_category_items', array( $this, 'ubp_load_box_category_items' ) );
			add_action( 'wp_ajax_ubp_load_box_category_items', array( $this, 'ubp_load_box_category_items' ) );
			add_filter( 'wc_ubp_before_product_addons', array( $this, 'display_categories_filter_on_front_side' ), 10, 4 );

			add_action( 'woocommerce_before_order_itemmeta', array( $this, 'action_woocommerce_before_order_itemmeta' ), 10, 3 );

			// Save cart item custom meta as order item meta data and display it everywhere on orders and email notifications.
			add_action( 'woocommerce_checkout_create_order_line_item', array( $this, 'save_cart_item_custom_meta_as_order_item_meta' ), 10, 4 );
			add_action( 'woocommerce_order_details_after_order_table_items', array( $this, 'custom_message_text' ), 10, 1 );
			add_action( 'woocommerce_admin_order_data_after_order_details', array( $this, 'custom_message_text' ), 10, 1 );

		}

		function custom_message_text( $order ) {
			echo '<div class="ubp_message_txt"></br>' . '<h3>' . esc_html__('Message', 'wc-ubp') . "</h3> <span>" . get_post_meta( $order->get_id(), 'ubp_box_message_field', true ) . '</span> </div>';
		}
		function save_cart_item_custom_meta_as_order_item_meta( $item, $cart_item_key, $values, $order ) {
			$meta_key = 'PR CODE';
			if ( isset( $values['ubp_box_message_field'] ) ) {
				$order->update_meta_data( 'ubp_box_message_field', $values['ubp_box_message_field'] );
			}
		}

		/*
		*Shows variation data variable product in box product on order details on admin side.
		*/
		public function action_woocommerce_before_order_itemmeta( $id, $order_item, $arg ) {
			if ( method_exists( $order_item, 'get_variation_id' ) && ! empty( $order_item->get_variation_id() ) ) {
				$product_item = wc_get_product( $order_item->get_variation_id() );

				if ( ( $product_item->is_type( 'variation' ) || $product_item->is_type( 'product_variation' ) ) && count( $product_item->get_attributes() ) > 2 ) {

					if ( get_option( '_ubp_box_product_bundled_link', 'yes' ) == 'yes' ) {
						$atts = $product_item->get_attributes();
						foreach ( $atts as $key => $value ) {
							$atts[ $key ] = '<strong>' . wc_attribute_label( $key ) . '</strong> - ' . $value;
						}
						echo '' . implode( '<br> ', $atts );
					}
				}
			}
		}

		/**
		 * This function will fetch more products of a provided category using ajax call.
		 */
		public function ubp_load_box_category_items() {
			 ob_start();
			include WOO_UBP_DIR . '/templates/cat_filter_products_more.php';
			$html = ob_get_clean();
			wp_send_json_success(
				array(
					'paged' => $paged,
					'html'  => $html,
				)
			);
		}

		/**
		 * This function will call when user will change the Category from front end side.
		 * This will call from ajax.
		 */
		public function ubp_caf_category_changed() {
			$posted_data = filter_input_array( INPUT_POST );
			$cats        = $posted_data['category_id'];
			$product_id  = $posted_data['product_id'];

			ob_start();
			include WOO_UBP_DIR . '/templates/cat_filter_products.php';
			$html = ob_get_clean();
			wp_send_json_success( $html );
		}

		/**
		 * This function will make the GUI for the category filter on front end side.
		 *
		 * @param string $html The markup that is using in the parent loop.
		 * @param array  $all_products Ids of all the products including addons and the categories.
		 * @param array  $only_addons  Ids of the addons only.
		 * @param array  $selected_categories The categories that are selected from the backend for this product.
		 * @return false|string|void
		 */
		public function display_categories_filter_on_front_side( $html, $all_products, $only_addons, $selected_categories ) {
			global $post;
			if ( ! $this->is_addon_enabled( $post->ID ) ) {
				return;
			}

			$category_setting = get_post_meta( $post->ID, 'ubp_category_addon_display_category_filter_setting', true );

			if ( ! empty( $only_addons ) && ( 'both' === $category_setting || 'only_products' === $category_setting ) ) {
				if ( empty( $selected_categories ) ) {
					$selected_categories = array();
				}

				// resetting as only product option is set.
				if ( 'only_products' === $category_setting ) {
					$selected_categories = array();
				}

				foreach ( $only_addons as $product_id ) {
					$term_list = wp_get_post_terms( $product_id, 'product_cat', array( 'fields' => 'ids' ) );
					foreach ( $term_list as $term_id ) {
						$selected_categories[] = $term_id;
					}
				}
			}

			$cat_data = array();
			if ( ! empty( $selected_categories ) ) {
				$selected_categories = array_unique( $selected_categories );
				foreach ( $selected_categories as $category_id ) {
					$term = get_term( $category_id, 'product_cat' );

					if ( strtolower( $term->name ) === 'uncategorized' ) {
						continue;
					}

					$cat_data[] = array(
						'id'   => $term->term_id,
						'name' => $term->name,
					);
				}
			}

			ob_start();
			?>

			<div class="categories-container form-group">
				<select class="form-control categories-selector" name="wc_ubp_category_selector" id="wc_ubp_category_selector">
					<option value=""><?php esc_html_e( 'All Categories', 'wc-ubp' ); ?></option>
					<?php foreach ( $cat_data as $category ) : ?>
						<option value="<?php echo esc_attr( $category['id'] ); ?>"><?php echo esc_attr( $category['name'] ); ?></option>
					<?php endforeach; ?>
				</select>
				<div class="ubp-loader-ripple ubp-loader-ripple-cat" style=" display: none; position: absolute;z-index: 9;left: 44%;">
					<div></div>
					<div></div>
				</div>
			</div>

			<?php
            return ob_get_clean(); // phpcs:ignore
		}

		/**
		 * This function will check if addon is enabled on specific product or its overriding the main functionality.
		 *
		 * @param null|integer $post_id The Post Id of the product.
		 * @return bool
		 */
		private function is_addon_enabled( $post_id = null ) {
			$is_enabled = esc_attr( get_option( 'ubp_is_category_filter_addon_active' ) );

			if ( null !== $post_id ) {
				$is_overriding = esc_attr( get_post_meta( $post_id, 'ubp_override_category_filter_addon', true ) );
				if ( 'yes' === $is_overriding ) {
					$is_enabled = esc_attr( get_post_meta( $post_id, 'ubp_is_category_filter_addon_active_for_product', true ) );
				}
			}

			return ( 'yes' === $is_enabled || '' === $is_enabled );
		}

		public function ubp_load_products() {
			if ( ! isset( $_POST['product_id'] ) || ! isset( $_POST['paged'] ) ) {
				wp_send_json_error( esc_html__( 'Sorry! the product id is missing.', 'wc-ubp' ) );
			}
			$product_id                = absint( $_POST['product_id'] );
			$paged                     = absint( $_POST['paged'] );
			$addons                    = get_post_meta( $product_id, 'ubp_products_addons_values', true );
			$addons                    = json_decode( $addons );
			$columns                   = get_post_meta( $product_id, 'ubp_products_columns', true );
			$qty                       = get_post_meta( $product_id, 'ubp_show_item_qty', true );
			$show_prod                 = get_option( 'ubp_mix_match_box_max_posts_per_page' );
			$show_prod                 = empty( $show_prod ) ? count( $addons ) : absint( $show_prod );
			$product_hide_out_of_stock = get_post_meta( $product_id, 'ubp_hide_out_of_stock_products', true );
			$qty_selector              = '';
			if ( $qty !== 'yes' ) {
				$qty_selector = 'style="display:none;opacity:0;"';
			}
			$cats_enabled = get_post_meta( $product_id, 'ubp_box_product_categories_enable', true );
			if ( empty( $_POST['category_id'] ) || $_POST['category_id'] == 0 ) {
				$cats = get_post_meta( $product_id, 'ubp_products_categories', true );
			} else {
				$cats = absint( $_POST['category_id'] );
			}

			$cat_prod_ids = array();
			if ( ! empty( $cats ) ) {
				$args         = array(
					'post_type'      => array( 'product' ),
					'posts_per_page' => -1,
					'fields'         => 'ids',
					'tax_query'      => array(
						array(
							'taxonomy' => 'product_cat',
							'field'    => 'term_id',
							'terms'    => $cats,
						),
					),
				);
				$cat_prod_ids = get_posts( $args );
			}

			if ( ! empty( $_POST['category_id'] ) && absint( $_POST['category_id'] ) != 0 ) {

				$dbcats           = get_post_meta( $product_id, 'ubp_products_categories', true );
				$category_setting = get_post_meta( $product_id, 'ubp_category_addon_display_category_filter_setting', true );
				if ( $category_setting === 'only_products' || $category_setting === 'both' ) {
					if ( ! in_array( $cats, $dbcats ) ) {
						$addon_ids = array();
						foreach ( $addons as $addon ) {
							$addon_ids[] = $addon->id;
						}

						$cat_prod_ids = array_intersect( $addon_ids, $cat_prod_ids );
					}
				}
			} else {
				$addons       = wp_list_pluck( (array) $addons, 'id' );
				$cat_prod_ids = array_unique( array_merge( $addons, $cat_prod_ids ) );
			}

			$all_prod = array();
			if ( $cats_enabled == 'yes' ) {

				if ( ! empty( $cat_prod_ids ) ) {
					$args          = array(
						'post_type'       => array( 'product_variation' ),
						'posts_per_page'  => -1,
						'fields'          => 'ids',
						'post_parent__in' => $cat_prod_ids,
					);
					$variation_ids = get_posts( $args );
					$all_prod      = array_merge( $variation_ids, $cat_prod_ids );
				}
			}
			$all_prod = array_unique( $all_prod, SORT_NUMERIC );
			$all_prod = apply_filters( 'ubp_box_product_addon_filter_variations', $all_prod );
			$all_prod = apply_filters( 'ubp_box_product_addon_disable_variable_products', $all_prod, $product_id );
			sort( $all_prod, SORT_NUMERIC );
			$arg = array(
				'post_type'      => array( 'product', 'product_variation' ),
				'post__in'       => $all_prod,
				'orderby'        => 'post__in',
				'posts_per_page' => $show_prod,
				'paged'          => $paged,
				'tax_query'      => array(
					'relation' => 'OR',
					array(
						'taxonomy' => 'product_type',
						'field'    => 'slug',
						'terms'    => array( 'simple', 'product_variation' ),
						'operator' => 'IN',
					),
					array(
						'taxonomy' => 'product_type',
						'operator' => 'NOT EXISTS',
					),
				),
			);
			if ( $product_hide_out_of_stock == 'yes' ) {
				$arg['meta_query'] = array(
					array(
						'key'     => '_stock_status',
						'value'   => 'instock',
						'compare' => '=',
					),
				);
			}

			$arg      = apply_filters( 'ubp_loop_frontend_query_args', $arg );
			$products = new WP_Query( $arg );
			$html     = '';

			if ( ! empty( $products ) ) {
				while ( $products->have_posts() ) {
					$products->the_post();
					$temp     = $itemHtml = '';
					$_product = wc_get_product( get_the_ID() );

					if ( ! $_product->is_type( 'variable' ) ) {
						$data                 = array();
						$data['id']           = $_product->get_id();
						$data['price']        = $_product->get_price();
						$data['purchaseable'] = ( $_product->is_in_stock() && $_product->is_purchasable() ) ? 1 : 0;
						$data['max']          = $_product->get_max_purchase_quantity();
						$disable              = ( $data['purchaseable'] == 1 ) ? '' : 'class="ubp-disabled"';
						$itemHtml            .= '<li data-product_id="' . esc_attr( $_product->get_id() ) . '" ' . ( $disable ) . '>';
						$itemHtml            .= '<figure>';
						if ( ! empty( $disable ) ) {
							$itemHtml .= '<span class="outofstock_prod_box">' . apply_filters( 'wc_ubp_box_product_outofstock_label', esc_html__( 'Out of stock', 'wc-ubp' ) ) . '</span>';
						} else {
							$itemHtml .= '<span class="add_prod_box"></span>';
						}
						if ( has_post_thumbnail( $_product->get_id() ) ) {
							$itemHtml .= wp_get_attachment_image( get_post_thumbnail_id( $_product->get_id() ), 'woocommerce_thumbnail' );
						} else {
							$itemHtml .= '<img src="' . WOO_UBP_URL . '/assets/images/placeholder.png" alt="product-image">';
						}
						$itemHtml .= '<div class="qty" ' . $qty_selector . '><input type="button" value="-" class="qtyminus" field="qty_' . esc_attr( $_product->get_id() ) . '" />
								<input type="text" name="qty_' . esc_attr( $_product->get_id() ) . '" value="1" >
								<input type="button" value="+" class="qtyplus" field="qty_' . esc_attr( $_product->get_id() ) . '" /></div>';
						$itemHtml .= '<figcaption><a href="' . esc_url( get_the_permalink( $_product->get_id() ) ) . '" target="_blank">' . esc_html( $_product->get_name() ) . '</a>';
						if ( get_post_meta( $product_id, 'ubp_show_item_pricing', true ) == 'yes' ) {
							$itemHtml .= '<span class="box_product_item_price">' . $_product->get_price_html() . '</span>';
						}
						$itemHtml .= '</figcaption>';
						$itemHtml .= '</figure>';
						$itemHtml .= '<input type="hidden" value="' . wc_esc_json( wp_json_encode( $data ) ) . '">';
						$itemHtml .= '</li>';
					}
					$html .= apply_filters( 'ubp_box_product_single_item_content', $itemHtml, $_product, wc_get_product( $product_id ) );
				}
			} else {
				$html .= esc_html__( 'Please add product addons for this box.', 'wc-ubp' );
			}
			if ( $products->max_num_pages > $paged ) {
				$paged++;
			} else {
				$paged = null;
			}
			wp_send_json_success(
				array(
					'paged' => $paged,
					'html'  => $html,
				)
			);
		}
	}
	new UBP_Woo_Box_Product_Controller();
}
