<?php
if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}
if (!class_exists('UBP_Variable_Support_Frontend')) {
	class UBP_Variable_Support_Frontend
	{
		public function __construct()
		{
			add_filter('ubp_box_product_single_item_content', array($this, 'vbp_box_product_single_item_content'), 10, 3);
			add_filter('upb_box_product_addons_options', array($this, 'vbp_product_addons_options'), 10, 1);
			add_filter('ubp_loop_frontend_query_args', array($this, 'vbp_frontend_query_args'), 10, 1);
			add_filter('ubp_loop_ajax_query_args', array($this, 'vbp_frontend_query_args'), 10, 1);
		}
		public function vbp_box_product_single_item_content($html, $_product, $product)
		{

			$temp = '';
			$qty = get_post_meta($product->get_id(), 'ubp_show_item_qty', true);
			$qty_selector = '';
			if ('yes' !== $qty) {
				$qty_selector = 'style="display:none;opacity:0;"';
			}

			if (($_product->is_type('variable') || $_product->get_parent_id() !== 0) && (!$_product->is_type('variation'))) {
				$data = array();
				$disable = ($_product->is_purchasable() && $_product->is_in_stock()) ? '' : 'class="ubp-disabled"';
				$html = '<li data-product_id="' . esc_attr($_product->get_id()) . '" ' . ($disable) . '>';
				$html .= '<figure>';
				if (!empty($disable))
					$html .= '<span class="outofstock_prod_box">' . apply_filters("wc_ubp_box_product_outofstock_label", esc_html__("Out of stock", "wc-ubp")) . '</span>';
				else
					$html .= '<span class="veriable_box_item">' . esc_html__('Choose variations', 'wc-vbp') . '</span>';

				if (has_post_thumbnail($_product->get_id())) {
					$html .= '<img src="' . esc_url(get_the_post_thumbnail_url($_product->get_id(), 'woocommerce_thumbnail')) . '"  alt="product-image">';
				} else {
					$html .= '<img src="' . WOO_UBP_URL . '/assets/images/placeholder.png" alt="product-image">';
				}
				$html .= '<div class="qty" ' . $qty_selector . '><input type="button" value="-" class="qtyminus" field="qty_' . esc_attr($_product->get_id()) . '" />
						<input type="text" name="qty_' . esc_attr($_product->get_id()) . '" value="1" >
						<input type="button" value="+" class="qtyplus" field="qty_' . esc_attr($_product->get_id()) . '" /></div>';
				$html .= '<figcaption><a href="' . esc_url(get_the_permalink($_product->get_id())) . '"  target="_blank">' . esc_html($_product->get_name()) . '</a>';
				if (get_post_meta($product->get_id(), 'ubp_show_item_pricing', true) == 'yes') {
					$html .= '<span class="box_product_item_price" target="_blank">' . $_product->get_price_html() . '</span>';
				}
				$html .= '</figcaption>';
				$html .= '</figure>';
				$html .= $this->vbp_print_variable_products_options($_product->get_id());
				$html .= '</li>';
			}
			return $html;
		}
		public function vbp_print_variable_products_options($product_id)
		{
			ob_start();
			woocommerce_variable_add_to_cart();
			$html = ob_get_clean();
			return $html;
		}
		public function vbp_product_addons_options($data)
		{
			if (!empty($data)) {
				foreach ($data as $key => $value) {
					if (is_numeric($key)) {
						$variable = wc_get_product($key);

						if (!is_wp_error($variable) && is_object($variable) && ($variable->is_type('variable') || $variable->is_type('variation'))) {

							if ($variable->is_type('variation')) {
								$parent_id = $variable->get_parent_id();
								$_variable = wc_get_product($parent_id);

								$this->set_variation($_variable, $data);
							} else {
								$this->set_variation($variable, $data);
							}
						}
					}
				}
			}
			return $data;
		}
		private function set_variation($variable, &$data)
		{
			if (is_object($variable) && method_exists($variable, 'get_available_variations')) {

				foreach ($variable->get_available_variations() as $key => $variation) {
					if ($variation['variation_is_active']) {
						$att = array();
						$max = ($variation['max_qty'] == '') ? -1 : $variation['max_qty'];
						$variation_id = $variation['variation_id'];
						$data[$variation_id] = array(
							'price'	=>	$variation['display_price'],
							'max'	=> ($variation['is_in_stock'] && $variation['is_purchasable']) ? $max : 0
						);
						unset($data[$variable->get_id()]);
					}
				}
			}
		}
		public function vbp_frontend_query_args($args)
		{
			$args['tax_query'] = array(
				'relation' => 'OR',
				array(
					'taxonomy' => 'product_type',
					'field'    => 'slug',
					'terms'    => array('simple', 'variable'),
					'operator' => 'IN',
				),
				array(
					'taxonomy' => 'product_type',
					'operator' => 'NOT EXISTS',
				)
			);
			if ('on' !== get_option('ubp_variable_product_addon_allow_variations')) {
				$args['post_type'] = array('product');
			}
			return $args;
		}
		public function vbp_enqueue_front_scripts()
		{
			$url_path = trim(parse_url(add_query_arg(array()), PHP_URL_PATH), '/');
			if (is_product() || strpos($url_path, 'edit-box-subscription/') !== false) {
				wp_enqueue_style('variable-box-style', WOO_UBP_URL . '/assets/css/frontend_style.css', array(), '');
				wp_enqueue_script('variable-box-script', WOO_UBP_URL . '/assets/js/frontend_script.js', array('jquery'), '');
			}
		}
	}
	new UBP_Variable_Support_Frontend();
}
